﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Data.SqlClient;
using C1.Win.C1FlexGrid;
using System.Globalization;
using System.Collections;
using System.Collections.Specialized;

namespace WindowsFormsApplication1
{
    public partial class Form1 : Form
    {
        private List<DictionaryEntry> category;
        public Form1()
        {
            InitializeComponent();
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            this.initializeCategory();
            this.initializeGrandTotal();
        }

        private void dateTimePicker1_ValueChanged(object sender, EventArgs e)
        {
            this.loadDatabase();
        }

        /// <summary>
        /// Returns the connection string of the database.
        /// </summary>
        /// <returns></returns>
        private string getConnectionString()
        {
            string connectionString = string.Empty;
            connectionString += "Data Source = (local)\\Developer2005;";
            connectionString += "Initial Catalog = CodeZine;";
            connectionString += "Integrated Security = SSPI;";

            return connectionString;
        }

        private void RegistrationButton_Click(object sender, EventArgs e)
        {
            this.writeDatabase();
        }

        private void ReadButton_Click(object sender, EventArgs e)
        {
            this.loadDatabase();
        }

        private void c1FlexGrid1_AfterEdit(object sender, C1.Win.C1FlexGrid.RowColEventArgs e)
        {
            this.calcGrossMargin(e.Row);
        }

        /// <summary>
        /// Set the intial value of aggregate row.
        /// </summary>
        private void initializeGrandTotal()
        {
            this.c1FlexGrid1.Tree.Column = 1;
            this.c1FlexGrid1.Tree.Style = TreeStyleFlags.Simple;

            // Clear the existing subtotal values.
            this.c1FlexGrid1.Subtotal(AggregateEnum.Clear);
            
            // Set the cell style for the grandtotal row.
            CellStyle cellStyle = this.c1FlexGrid1.Styles[CellStyleEnum.GrandTotal];
            cellStyle.BackColor = Color.Blue;
            cellStyle.ForeColor = Color.White;
            cellStyle.Font = new Font(Font, FontStyle.Bold);

            // Set the cell style for subtotal row.
            cellStyle = this.c1FlexGrid1.Styles[CellStyleEnum.Subtotal0];
            cellStyle.BackColor = Color.Orchid;
            cellStyle.ForeColor = Color.White;
            cellStyle.Font = new Font(Font, FontStyle.Bold);
        }

        /// <summary>
        /// Set the intial values for Category.
        /// </summary>
        private void initializeCategory()
        {
            SqlConnection connection = new SqlConnection(this.getConnectionString());

            // Open database connection.
            connection.Open();

            // Generate an instance of SqlCommand from SqlConnection.
            SqlCommand command = connection.CreateCommand();

            // Set the SQL command to be executed.
            command.CommandText = "SELECT CategoryCode, CategoryName FROM CategoryMaster ORDER BY CategoryCode";

            // Run the SQL command and create SqlDataReader.
            SqlDataReader reader = command.ExecuteReader();

            // Dispose command.
            command.Dispose();

            this.category = new List<DictionaryEntry>();

            // Go to the next record
            while (reader.Read())
            {
                this.category.Add(new DictionaryEntry(reader["CategoryCode"], reader["CategoryName"]));
            }

            // Close reader.
            reader.Close();

            // Close database connection.
            connection.Close();
            connection.Dispose();

            ListDictionary dictionary = new ListDictionary();

            foreach(DictionaryEntry entry in this.category )
            {
                dictionary.Add(entry.Key, entry.Value);
            }

            this.c1FlexGrid1.Cols[1].DataType = typeof(int);
            this.c1FlexGrid1.Cols[1].DataMap = dictionary;
        }

        /// <summary>
        /// Display the aggregate row.
        /// </summary>
        private void showTotal()
        {
            // Clear subtotal.
            this.c1FlexGrid1.Subtotal(AggregateEnum.Clear);

            // Sort the contents of the grid.
            this.c1FlexGrid1.Sort(SortFlags.Ascending, 0, 1);

            // Calculate the grandtotal.
            this.c1FlexGrid1.Subtotal(AggregateEnum.Sum, -1, -1, 2, "Total");
            this.c1FlexGrid1.Subtotal(AggregateEnum.Sum, -1, -1, 3, "Total");
            this.c1FlexGrid1.Subtotal(AggregateEnum.Sum, -1, -1, 4, "Total");
            this.c1FlexGrid1.Subtotal(AggregateEnum.Average, -1, -1, 5, "Total");

            // Calculate the subtotal.
            this.c1FlexGrid1.Subtotal(AggregateEnum.Sum, 0, 0, 2, "{0:MM/dd} Subtotal");
            this.c1FlexGrid1.Subtotal(AggregateEnum.Sum, 0, 0, 3, "{0:MM/dd} Subtotal");
            this.c1FlexGrid1.Subtotal(AggregateEnum.Sum, 0, 0, 4, "{0:MM/dd} Subtotal");
        }

        /// <summary>
        /// Print the contents of the grid.
        /// </summary>
        private void printGrid()
        {
            // Set page settings for printing (Portrait).
            this.c1FlexGrid1.PrintParameters.PrintDocument.DefaultPageSettings.Landscape = false;

            // Set margin on the left of the document.
            this.c1FlexGrid1.PrintParameters.PrintDocument.DefaultPageSettings.Margins.Left = 150;

            // Set header and footer font.
            this.c1FlexGrid1.PrintParameters.HeaderFont
                = new Font("Arial Narrow", 10, FontStyle.Regular);
            this.c1FlexGrid1.PrintParameters.FooterFont
                = new Font("Arial Narrow", 8, FontStyle.Italic);

            // Get the preview of the grid.
            string title = string.Format("Sales Data for {0}", CultureInfo.CurrentCulture.DateTimeFormat.GetMonthName(this.dateTimePicker1.Value.Month));
            this.c1FlexGrid1.PrintGrid(title,
                PrintGridFlags.FitToPage | PrintGridFlags.ShowPreviewDialog,
                title,
                (char)(9) + "" + (char)(9) + "Page {0}/{1}");
        }

        private void readToolStripMenuItem_Click(object sender, EventArgs e)
        {
            this.loadDatabase();
        }

        private void registrationToolStripMenuItem_Click(object sender, EventArgs e)
        {
            this.writeDatabase();
        }

        private void printToolStripMenuItem_Click(object sender, EventArgs e)
        {
            this.printGrid();
        }

        /// <summary>
        /// Write values to the database.
        /// </summary>
        private void writeDatabase()
        {
            SqlConnection connection = new SqlConnection(this.getConnectionString());

            // Open database connection.
            connection.Open();

            SqlCommand command = new SqlCommand();
            command.Connection = connection;
            command.CommandType = CommandType.Text;

            // Delete data.
            DateTime selectedMonth
                = new DateTime(this.dateTimePicker1.Value.Year, this.dateTimePicker1.Value.Month, 1);

            command.CommandText =
                string.Format("DELETE FROM Sales WHERE Date >= '{0}' AND Date < '{1}'",
                selectedMonth,
                selectedMonth.AddMonths(1));
            command.ExecuteNonQuery();

            for (int i = 1; i < this.c1FlexGrid1.Rows.Count - 1; i++)
            {
                if ( this.c1FlexGrid1[i, 0] != null 
                    && (this.c1FlexGrid1[i, 2] != null
                    || this.c1FlexGrid1[i, 3] != null) )
                {
                    string insert =
                        string.Format("INSERT Sales (Date, CategoryCode, Proceeds, Payments, GrossMargin, GrossMarginRate) VALUES ('{0}', {1}, {2}, {3}, {4}, {5})",
                        this.c1FlexGrid1[i, 0],
                        this.c1FlexGrid1[i, 1] == null ? 0 : this.c1FlexGrid1[i, 1],
                        this.c1FlexGrid1[i, 2] == null ? 0 : this.c1FlexGrid1[i, 2],
                        this.c1FlexGrid1[i, 3] == null ? 0 : this.c1FlexGrid1[i, 3],
                        this.c1FlexGrid1[i, 4],
                        this.c1FlexGrid1[i, 5]);

                    command.CommandType = CommandType.Text;
                    command.CommandText = insert;
                    command.ExecuteNonQuery();
                }
            }

            // Dispose command.
            command.Dispose();

            // Close database connection.
            connection.Close();
            connection.Dispose();
        }

        /// <summary>
        /// Get the value of the database.
        /// </summary>
        private void loadDatabase()
        {
            SqlConnection connection = new SqlConnection(this.getConnectionString());

            // Open database connection.
            connection.Open();

            // Generate an instance of SqlCommand from SqlConnection.
            SqlCommand command = connection.CreateCommand();

            DateTime selectedMonth
                = new DateTime(this.dateTimePicker1.Value.Year, this.dateTimePicker1.Value.Month, 1);

            string selectSQL
                = string.Format("SELECT Date, CategoryCode, Proceeds, Payments, GrossMargin, GrossMarginRate FROM Sales"
                                    + " WHERE Date >= '{0}' AND Date < '{1}'"
                                    + " ORDER BY Date, CategoryCode", selectedMonth, selectedMonth.AddMonths(1));

            // Set the SQL command to be executed.
            command.CommandText = selectSQL;

            // Run the SQL command and create SqlDataReader.
            SqlDataReader reader = command.ExecuteReader();

            // Dispose command.
            command.Dispose();

            this.c1FlexGrid1.Rows.RemoveRange(1, this.c1FlexGrid1.Rows.Count - 1);

            // Go to the next record.
            while (reader.Read())
            {
                this.c1FlexGrid1.AddItem(new object[] { reader["Date"], reader["CategoryCode"], reader["Proceeds"], reader["Payments"], reader["GrossMargin"], reader["GrossMarginRate"] });
            }

            // Close reader.
            reader.Close();

            // Close database connection.
            connection.Close();
            connection.Dispose();
            this.showTotal();
        }

        private void c1FlexGrid1_OwnerDrawCell(object sender, OwnerDrawCellEventArgs e)
        {
            // Settings meant for GrossMarginRate column only (exclude fixed row and aggregate row).
            if (e.Col == 5 && e.Row >= c1FlexGrid1.Rows.Fixed && e.Row != c1FlexGrid1.Rows.Count -1)
            {
                double value;
                if (double.TryParse(c1FlexGrid1.GetDataDisplay(e.Row, e.Col), NumberStyles.Any, CultureInfo.CurrentCulture, out value))
                {
                    // Calculate horizontal width.
                    //Rectangle rc = e.Bounds;
                    Rectangle rc = this.c1FlexGrid1.GetCellRect(e.Row, 5);
                    rc.Width = (int)(this.c1FlexGrid1.Cols[e.Col].WidthDisplay * value);

                    // Draw the background of the cell.
                    e.DrawCell(DrawCellFlags.Background | DrawCellFlags.Border);

                    // Draw a bar graph.
                    rc.Inflate(-1, -1);
                    e.Graphics.FillRectangle(Brushes.LimeGreen, rc);
                    rc.Inflate(-1, -1);
                    e.Graphics.FillRectangle(Brushes.PaleGreen, rc);

                    // Draw the contents of the cell.
                    e.DrawCell(DrawCellFlags.Content);
                }
            }
        }

        private void c1FlexGrid1_DoubleClick(object sender, EventArgs e)
        {
            if (this.c1FlexGrid1.Row < 0 || this.c1FlexGrid1.GetDataDisplay(this.c1FlexGrid1.Row, 0).Length == 0)
            {
                return;
            }

            InputForm input = new InputForm();
            input.Category = this.category;
            int rowNum = this.c1FlexGrid1.Row;

            input.Date = DateTime.Parse( this.c1FlexGrid1[rowNum, 0].ToString() );
            input.CategoryCode = this.c1FlexGrid1[rowNum, 1];
            input.Proceeds = this.c1FlexGrid1[rowNum, 2] == null ? 0 : int.Parse(this.c1FlexGrid1[rowNum, 2].ToString());
            input.Payments = this.c1FlexGrid1[rowNum, 3] == null ? 0 : int.Parse(this.c1FlexGrid1[rowNum, 3].ToString());

            if (input.ShowDialog() == DialogResult.OK)
            {
                this.c1FlexGrid1[rowNum, 0] = input.Date.Date;
                this.c1FlexGrid1[rowNum, 1] = input.CategoryCode;
                this.c1FlexGrid1[rowNum, 2] = input.Proceeds;
                this.c1FlexGrid1[rowNum, 3] = input.Payments;
            }
            input.Dispose();

            this.calcGrossMargin(rowNum);
        }
        
        /// <summary>
        /// Calculate gross margin.
        /// </summary>
        /// <param name="rowNum"></param>
        private void calcGrossMargin(int rowNum)
        {
            double proceeds = this.c1FlexGrid1[rowNum, 2] == null
                    ? 0 : double.Parse(this.c1FlexGrid1[rowNum, 2].ToString());
            double payments = this.c1FlexGrid1[rowNum, 3] == null
                    ? 0 : double.Parse(this.c1FlexGrid1[rowNum, 3].ToString());
            double grossMargin = proceeds - payments;
            this.c1FlexGrid1[rowNum, 4] = grossMargin;
            this.c1FlexGrid1[rowNum, 5] = proceeds != 0 ? grossMargin / proceeds : 0;

            this.showTotal();
        }

        private void addButton_Click(object sender, EventArgs e)
        {
            this.addData();
        }

        private void addToolStripMenuItem_Click(object sender, EventArgs e)
        {

            this.addData();
        }

        /// <summary>
        /// Add the data.
        /// </summary>
        private void addData()
        {
            InputForm input = new InputForm();
            input.Category = this.category;

            if (input.ShowDialog() == DialogResult.OK)
            {
                int rowNumber = this.c1FlexGrid1.AddItem(new object[] { input.Date.Date, input.CategoryCode, input.Proceeds, input.Payments }).Index;
                this.calcGrossMargin(rowNumber);
            }
            input.Dispose();
        }

    }
}   
